package com.ruoyi.agent.service.impl;

import com.ruoyi.agent.service.IMcpService;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.noear.solon.ai.annotation.PromptMapping;
import org.noear.solon.ai.annotation.ResourceMapping;
import org.noear.solon.ai.annotation.ToolMapping;
import org.noear.solon.ai.chat.message.ChatMessage;
import org.noear.solon.ai.mcp.server.annotation.McpServerEndpoint;
import org.noear.solon.annotation.Param;
import org.springframework.stereotype.Service;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

@Service
@McpServerEndpoint(sseEndpoint = "/llm/mcp/sse")
public class McpServiceImpl implements IMcpService {

    @ToolMapping(description = "查询天气预报")
    public String getWeather(@Param(description = "城市位置") String location) {
        return "晴，14度";
    }

    @ToolMapping(description = "查看招标文档")
    public String openDocument(@Param(description = "文件路径") String document) throws IOException {
        try (XWPFDocument doc = new XWPFDocument(new FileInputStream(document))) {
            StringBuilder content = new StringBuilder();
            for (XWPFParagraph paragraph : doc.getParagraphs()) {
                content.append(paragraph.getText()).append("\n");
            }
            return content.toString();
        }
    }

    @ToolMapping(description = "补全技术方案")
    public String writeTechnicalPlan(@Param(description = "文件路径") String document, @Param(description = "技术方案内容") String resultContent) throws IOException {
        // 创建临时输出文件路径
        String outputPath = document.replace(".docx", "_modified.docx");

        try (FileInputStream fis = new FileInputStream(document);
             XWPFDocument doc = new XWPFDocument(fis);
             FileOutputStream fos = new FileOutputStream(outputPath)) {

            // 在最后添加分页符和新内容
            XWPFParagraph pageBreak = doc.createParagraph();
            pageBreak.setPageBreak(true);

            XWPFParagraph newContent = doc.createParagraph();
            newContent.createRun().setText(resultContent);

            // 保存文档
            doc.write(fos);

            return "技术方案添加完成，已保存到: " + outputPath;
        } catch (IOException e) {
            throw new IOException("处理Word文档时出错: " + e.getMessage(), e);
        }
    }

    @ResourceMapping(uri = "config://app-version", description = "获取应用版本号")
    public String getAppVersion() {
        return "v3.2.0";
    }

    @ResourceMapping(uri = "db://users/{user_id}/email", description = "根据用户ID查询邮箱")
    public String getEmail(@Param(description = "用户Id") String user_id) {
        return user_id + "@example.com";
    }

    @PromptMapping(description = "提出技术问题")
    public Collection<ChatMessage> askQuestion(@Param(description = "描述招标文件服务要求") String request) {
        return Collections.singletonList(
                ChatMessage.ofUser("请根据以下要求：'" + request + "\n编写技术方案")
        );
    }
}