package com.ruoyi.web.llm.controller;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.web.llm.service.ILangChainMilvusService;
import org.noear.solon.ai.chat.ChatModel;
import org.noear.solon.ai.chat.ChatResponse;
import org.noear.solon.ai.chat.message.AssistantMessage;
import org.noear.solon.ai.chat.message.ChatMessage;
import org.noear.solon.ai.chat.tool.FunctionToolDesc;
import org.noear.solon.ai.mcp.client.McpClientProvider;
import org.reactivestreams.Publisher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Flux;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * mcp模型上下文协议Controller
 * 
 * @author cmc
 * @date 2025-04-08
 */
@RestController
@RequestMapping("/llm/mcp")
public class McpController extends BaseController
{
    @Autowired
    private ILangChainMilvusService langChainMilvusService;

    /**
     * 打开word文档
     * @return
     */
    @GetMapping("/answer")
    public Flux<AssistantMessage> answer(String question) throws IOException {
        McpClientProvider clientProvider = McpClientProvider.builder()
                    .apiUrl("http://localhost:8080/llm/mcp/sse")
                    .build();

        ChatModel chatModel = ChatModel.of("http://192.168.28.188:8000/v1/chat/completions")
                .provider("openai")
                .model("DeepSeek-R1-Distill-Qwen-1.5B")
                .apiKey("1")
                .defaultToolsAdd(clientProvider)
                .build();
        Publisher<ChatResponse> publisher = chatModel.prompt(question).stream();

        return Flux.from(publisher)
                .map(response -> {
                    return response.getChoices().get(0).getMessage();
                });
    }

    /**
     * 根据招标文件要求编写投标文件技术方案
     * @return
     */
    @GetMapping("/tech")
    public AssistantMessage tech(String file) throws IOException {
        McpClientProvider clientProvider = McpClientProvider.builder()
                    .apiUrl("http://localhost:8080/llm/mcp/sse")
                    .build();
        ChatModel chatModel = ChatModel.of("http://192.168.28.188:8000/v1/chat/completions")
                .provider("openai")
                .model("DeepSeek-R1-Distill-Qwen-1.5B")
                .apiKey("1")
                .defaultToolsAdd(clientProvider)
                .build();

        Map<String,Object> path = new HashMap<>();
        path.put("document", file);
        String content = clientProvider.callToolAsText("openDocument", path).getContent();
        Map<String,Object> request = new HashMap<>();
        request.put("request", content);
        List<ChatMessage> messages = clientProvider.getPromptAsMessages("askQuestion", request);
        ChatResponse response = chatModel.prompt(messages).call();
        System.out.println(response.getChoices().get(0).getMessage());
        Map<String,Object> write = new HashMap<>();
        write.put("document", file);
        write.put("resultContent", response.getChoices().get(0).getMessage().getResultContent());
        clientProvider.callToolAsText("writeTechnicalPlan", write);
        return response.getChoices().get(0).getMessage();
    }

}
